var Q = require('q');
var NomieCommandActions = require('./commands.js').actions;

/**
 * Export the NomieCommand as function
 * Usage: var command = new NomieCommand();
 * command.init('nomie://action=track/label=Peed')...
 * @return {function}
 */
module.exports = function() {

  // Set Public and Private Methods
  var pub = {};
  var pvt = {
    command: null
  }
  // Assign all available actions
  pub.available = NomieCommandActions;

  /**
   * Convert Command to String
   * @return {string} string representation of the command
   */
  pub.toString = function() {
    return pvt.stringify(pvt.command);
  };

  /**
   * COnvert Command to Object
   * @return {object} Returns filtered and approved object from a command
   */
  pub.toObject = function() {
    return pvt.command;
  }

  /**
   * INitialize a Command
   * @param  {any} objOrStr String or an Object
   * @return {object}          Self
   */
  pub.init = function(objOrStr) {
    pvt.origObj = objOrStr;
    pvt.command = {};
    var obj = {};
    if (typeof objOrStr === 'string') {
      pvt.command = pvt.transform(pvt.parse(objOrStr));
    } else {
      pvt.command = pvt.transform(objOrStr);
    }

    return pub;
  };

  /**
   * Private Commands that are Valid
   * @type {Array}
   */
  //console.log("Valid caommands", Object.keys(NomieCommandActions))
  pvt.validCommands = Object.keys(NomieCommandActions);

  /**
   * Validate a String
   * @param  {[type]} command [description]
   * @return {[type]}         [description]
   */
  pvt.getDefault = function(theDefault) {
    //console.log("### GETTING DEFAULT", theDefault, typeof theDefault);
    if (typeof theDefault != 'function') {
      return theDefault;
    } else {
      return theDefault();
    }
  }

  /**
   * Format the Valid
   * @param  {object} value Value to process
   * @param  {string} type  integer, float or string
   * @return {object}       Could be whatever it is
   */
  pvt.getValue = function(value, type) {
    if (!type) {
      return value;
    } else if (type == 'integer') {
      return parseInt(value);
    } else if (type == 'float') {
      return parseFloat(value);
    } else {
      return value;
    }
  }

  /**
   * Transform a command - make it clean and presitine
   * @param  {object} command Command Object
   * @return {object}         CLeaned Object
   */
  pvt.transform = function(command) {
    var errors = [];
    // Does it have an action
    if (!NomieCommandActions.hasOwnProperty(command.action)) {
      errors.push("Invalid Action Name: " + command.action);
    } else {
      // Find the action from the Stubs
      var ActionStub = NomieCommandActions[command.action];
      // Loop over action props
      for (var i in ActionStub.props) {
        var key = i;
        var prop = ActionStub.props[i];
        prop.type = prop.type || "string";

        // If its required AND command[key] doesnt exist AND There's a Default
        if (prop.required && !command.hasOwnProperty(key) && pvt.getDefault(prop.default)) {
          command[key] = (pvt.getDefault(prop.default) || "");

        // Else if its require AND command[key] doesnt exist
        } else if (prop.required && !command.hasOwnProperty(key)) {
          errors.push(key + " is required but missing");

        // Else if There's a default AND No command[key]
        } else if (pvt.getDefault(prop.default) && !command.hasOwnProperty(key)) {
          command[key] = (pvt.getDefault(prop.default) || "");

        } else if(command.hasOwnProperty(key)) {
          command[key] = (pvt.getValue(command[key], prop.type) || "");
        }
      }
    }

    // Any errors?
    if (!errors.length) {
      return command;
    } else {
      command.valid = false;
      command.success = false;
      command.errors = errors;
      return command;
    }
  };

  /**
   * Scrub the Command of common stuff
   * @param  {string} str String
   * @return {string}     Removed of stuffs
   */
  pvt.scrub = function(str) {
    return str.replace('nomie://', '').replace('valid=true/', '');
  }

  /**
   * Parse a Command String
   * @param  {string} commandStr
   * @return {[type]}            [description]
   */
  pvt.parse = function(commandStr) {
    commandStr = pvt.scrub(commandStr);
    var elements = {};
    var message = null;
    elements.valid = false;
    var base = commandStr.split('/');
    for (var i in base) {
      var keyvalue = base[i].split("=");
      if (keyvalue.length == 2) {
        elements[keyvalue[0]] = decodeURIComponent(keyvalue[1]);
      }
    }
    if (elements.hasOwnProperty('action')) {
      if (pvt.validCommands.indexOf(elements.action) > -1) {
        elements.valid = true;
      } else {
        message = elements.action + " is not a valid command";
      }
    } else {
      message = "Missing action or _d - not a valid request";
    }

    if (elements.valid == true) {
      //console.log("Valid Elements", elements);
      return elements;
    } else {
      return {
        valid: false,
        message: message
      }
    }
  }

  /**
   * Create a Stringify version of a nomie command
   * @param  {string} name   Name of the action
   * @param  {object} params Object of parameters
   * @return {string}        String representation
   */
  pvt.stringify = function(commandObj) {
    var command = [];
    for (var i in commandObj) {
      var param = [];
      param.push(i);
      param.push(encodeURI(commandObj[i]))
      command.push(param.join("="));
    }
    //console.log("## COMMAND ## STRINGIFY", commandObj, 'nomie://'+command.join('/'));
    var commandStr = pvt.scrub(command.join('/'));

    return 'nomie://' + commandStr;
  }

  return pub;
};

/**
 * If this is a Window request, set it
 */
if (typeof window !== 'undefined') {
  window.NomieCommand = module.exports;
}
